"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const index_1 = require("../index");
describe('to result', () => {
    it('not an object throws an error', () => {
        expect.assertions(2);
        expect(() => {
            (0, index_1.toResult)(null);
        }).toThrow('Unexpected input');
        expect(() => {
            (0, index_1.toResult)('123');
        }).toThrow('Unexpected input');
    });
    it('no status will throws an error', () => {
        expect.assertions(1);
        expect(() => {
            (0, index_1.toResult)({ data: 1 });
        }).toThrow('Unexpected input');
    });
    it('will cast to success', () => {
        expect.assertions(1);
        const result = (0, index_1.toResult)({ status: 'success', data: 1 });
        expect(result.ok()).toStrictEqual(1);
    });
    it('will cast to error', () => {
        expect.assertions(1);
        const result = (0, index_1.toResult)({ status: 'error', error: { type: '' } });
        expect(result.err()).toStrictEqual({ type: '' });
    });
    it('extra error data is added correctly', () => {
        expect.assertions(1);
        const result = (0, index_1.toResult)({
            status: 'error',
            error: { type: 'error added', someErrorData: 'err' },
        });
        expect(result.err()).toStrictEqual({ type: 'error added', someErrorData: 'err' });
    });
    it('will fail to unexpected status', () => {
        expect.assertions(1);
        expect(() => {
            return (0, index_1.toResult)({ status: 'wrong', data: 1, error: { type: '' } });
        }).toThrow('Unexpected input');
    });
    it('will cast to error if no error', () => {
        expect.assertions(4);
        expect((0, index_1.toResult)({ status: 'error' }).err()).toBeUndefined();
        expect((0, index_1.toResult)({ status: 'error', error: null }).err()).toBeNull();
        expect((0, index_1.toResult)({ status: 'error', error: 1 }).err()).toStrictEqual(1);
        expect((0, index_1.toResult)({ status: 'error', error: {} }).err()).toStrictEqual({});
    });
    it('handle success types', () => {
        expect.assertions(1);
        const result = (0, index_1.toResult)({
            status: 'success',
            data: { output: 'output' },
        });
        expect(result.ok().output).toStrictEqual('output');
    });
    it('handle error types', () => {
        expect.assertions(2);
        const result = (0, index_1.toResult)({
            status: 'error',
            error: { type: 'e1', someErrorData: 'err' },
        });
        expect(result.err().type).toStrictEqual('e1');
        expect(result.err().someErrorData).toStrictEqual('err');
    });
});
//# sourceMappingURL=cast.js.map