"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const index_1 = require("../index");
describe('util', () => {
    it('works with skip option', () => {
        expect.assertions(2);
        const r1 = (0, index_1.ok)('', { skip: true });
        const r2 = (0, index_1.fail)('', { skip: true });
        expect(r1.order).toStrictEqual(-Infinity);
        expect(r2.order).toStrictEqual(-Infinity);
    });
    it('compare equal results', () => {
        expect.assertions(2);
        const r1 = (0, index_1.ok)('');
        const r2 = (0, index_1.fail)('');
        expect((0, index_1.compare)(r1, r2)).toStrictEqual(r2);
        const r3 = (0, index_1.fail)('');
        const r4 = (0, index_1.fail)('');
        expect((0, index_1.compare)(r3, r4)).toStrictEqual(r3);
    });
    it('compare not equal results', () => {
        expect.assertions(2);
        const r1 = (0, index_1.fail)('', { order: 1 });
        const r2 = (0, index_1.fail)('', { order: 2 });
        expect((0, index_1.compare)(r1, r2)).toStrictEqual(r2);
        const r3 = (0, index_1.fail)('', { order: 2 });
        const r4 = (0, index_1.fail)('', { order: 1 });
        expect((0, index_1.compare)(r3, r4)).toStrictEqual(r3);
    });
    it('undefined error type must not have error object', () => {
        expect.assertions(1);
        const r = (0, index_1.err)(undefined);
        expect(r.err()).toBeUndefined();
    });
    it('converting result to string', () => {
        expect.assertions(3);
        const r1 = (0, index_1.ok)('1');
        expect(r1.toString()).toStrictEqual('Success<"1">');
        const r2 = (0, index_1.fail)('1');
        expect(r2.toString()).toStrictEqual('Failure<{"type":"1"}>');
        r2.status = 'any';
        expect(() => {
            r2.toString();
        }).toThrow('Unknown status');
    });
    it('throw fail', () => {
        expect.assertions(2);
        expect(() => {
            (0, index_1.throwFail)('Thrown');
        }).toThrow('Thrown');
        expect(() => {
            const a = Math.random() === -1 ? 'never' : undefined;
            if (!a) {
                (0, index_1.throwFail)('Thrown');
            }
            const b = a;
            expect(b).toStrictEqual('never');
        }).toThrow('Thrown');
    });
    it('never will throw exception if called', () => {
        expect.assertions(1);
        expect(() => {
            (0, index_1.nope)(undefined);
        }).toThrow('Unreachable');
    });
});
//# sourceMappingURL=util.js.map